<?php
/**
 * Created by PhpStorm.
 * User: jderosa
 * Date: 30/10/19
 * Time: 20.03
 */

namespace FiloBlu\Rma\Setup;
use Magento\Framework\Setup\UpgradeDataInterface;
use Magento\Framework\Setup\ModuleDataSetupInterface;
use Magento\Framework\Setup\ModuleContextInterface;
use FiloBlu\Rma\Helper\InstallHelper;
use Symfony\Component\Console\Output\ConsoleOutput as Output;
use Magento\Cms\Model\BlockFactory;
use Magento\Framework\App\State;
use Magento\Framework\App\ResourceConnection;
use Magento\Store\Api\StoreRepositoryInterface;
use Magento\Framework\App\Config\ScopeConfigInterface;


class UpgradeData implements UpgradeDataInterface
{
    /**
     * @var Output
     */
    protected $output;

    /**
     * @var InstallHelper
     */
    protected $installHelper;
    /**
     * @var BlockFactory
     */
    private $blockFactory;

    /**
     * @var State
     */
    private $state;
    /**
     * @var ResourceConnection
     */
    private $resourceConnection;

    const CMS_BLOCK_IDENTIFIER ='rma_logistic_warning';

    /**
     * @var ScopeConfigInterface
     */
    private $scopeConfig;

    private static $contents =
        [
            'it' =>'',
            'en' =>'',
            'de' =>'',
            'ja' =>'',
            'ru' =>'',
            'fr' =>'',
            'es' =>'',
            'default' =>''
        ];
    /**
     * @var StoreRepositoryInterface
     */
    private $storeRepositoryInterface;


    /**
     * UpgradeData constructor.
     * @param Output $output
     * @param InstallHelper $installHelper
     * @param BlockFactory $blockFactory
     * @param State $state
     * @param ResourceConnection $resourceConnection
     * @param StoreRepositoryInterface $storeRepositoryInterface
     * @param ScopeConfigInterface $scopeConfig
     */
    public function __construct(
        Output $output,
        InstallHelper $installHelper,
        BlockFactory $blockFactory,
        State $state,
        ResourceConnection $resourceConnection,
        StoreRepositoryInterface $storeRepositoryInterface,
        ScopeConfigInterface $scopeConfig)
    {
        $this->output = $output;
        $this->installHelper = $installHelper;
        $this->blockFactory = $blockFactory;
        $this->state = $state;
        $this->resourceConnection = $resourceConnection;
        $this->scopeConfig = $scopeConfig;
        $this->storeRepositoryInterface = $storeRepositoryInterface;
    }

    /**
     * @param ModuleDataSetupInterface $setup
     * @param ModuleContextInterface $context
     */
    public function upgrade(ModuleDataSetupInterface $setup, ModuleContextInterface $context)
    {
        try {
            $setup->startSetup();
            if (version_compare($context->getVersion(), "2.0.1", "<")) {
                $this->installHelper->addNewOrderStatus($setup);
                $this->installHelper->addNewReasonOption($setup);
                $this->installHelper->addNewRmaItem($setup);
                $this->installHelper->hideReasonOther($setup);
                $this->installHelper->sortAttributes($setup);
            }
            if (version_compare($context->getVersion(), "2.2.0", "<")) {
                //install easy return cms block
                if(!$this->checkIfCmsExists(self::CMS_BLOCK_IDENTIFIER)){
                    //build unique cms block grouped by locale

                    $this->state->emulateAreaCode(\Magento\Framework\App\Area::AREA_ADMINHTML,function(){
                        $groupedCmsBlock = $this->buildCmsBlocksUsingLocale();
                        foreach ($groupedCmsBlock as $locale => $cms)
                        {
                            $cmsBlock = $this->blockFactory->create();
                            $cmsBlockData =
                                [
                                    'title'=> "RMA warnings for language ".$locale,
                                    'identifier' => self::CMS_BLOCK_IDENTIFIER,
                                    'stores' => explode(",",$cms),
                                    'content' => $this->getContentByLocale($locale)

                                ];
                            $cmsBlock->setData($cmsBlockData);
                            $cmsBlock->save();
                        }

                    });

                }

            }
            //install Be standard reason options
            if (version_compare($context->getVersion(), "2.2.1", "<=")) {
                $this->installHelper->createBeReturnReasons($setup);
            }

            $setup->endSetup();
        }catch (\Exception $e) {
            $this->output->writeln("Error while creating new order status : ".$e->getMessage());
            $setup->endSetup();
        }catch (\Throwable $t){
            $this->output->writeln("Error while creating new order status : ".$t->getMessage());
            $setup->endSetup();
        }
    }

    /**
     * @param $identifier
     * @return bool
     */
    private function checkIfCmsExists($identifier)
    {
        $connection = $this->resourceConnection->getConnection();
        $table = $connection->getTableName('cms_block');
        $row = $connection->fetchRow("SELECT * FROM " . $table . " WHERE identifier = :identifier", ["identifier" => $identifier]);
        if ($row) {
            return true;
        } else {
            return false;
        }
    }

    /**
     * @return \Magento\Store\Api\Data\StoreInterface[]
     */
    private function getAllStoreViews(){
        return $this->storeRepositoryInterface->getList();
    }


    /**
     * @param int $storeId
     * @return mixed
     */
    private function getStoreLocaleByStoreId(int $storeId){
        $scopeConfig = $this->scopeConfig;
        $localeCode = $scopeConfig->getValue('general/locale/code',\Magento\Store\Model\ScopeInterface::SCOPE_STORE,$storeId);
        return $localeCode;
    }


    /**
     * @param string $locale
     * @return mixed|string
     */
    private function getContentByLocale(string $locale){
        foreach (self::$contents as $k => $content){
            if(strstr($locale,$k)){
                return $content;
            }
        }
        return self::$contents['default'];
    }


    /**
     * @return array
     */
    private function buildCmsBlocksUsingLocale(){
        $allStores = $this->getAllStoreViews();
        $cmsBlock = [];
        foreach ($allStores as $store){
            if($store->getCode()!="admin"){
                $locale = $this->getStoreLocaleByStoreId($store->getId());
                if(!isset($cmsBlock[$locale])){
                    $cmsBlock[$locale] = $store->getId();
                }else{
                    $cmsBlock[$locale] = $cmsBlock[$locale] . ",".$store->getId();
                }
            }


        }
        return $cmsBlock;
    }

}